# -*- coding: utf-8 -*-
import io

from odoo import fields
from odoo.addons.saas_worker.controllers import main
from odoo.addons.saas_worker.util import version_gte
from odoo.addons.saas_trial.tests.common import TestTrialCommon
from odoo.tests import tagged
from odoo.tools import load_language, mute_logger

try:
    # >= 16.0
    from odoo.tools.translate import TranslationImporter
except ImportError:
    from odoo.tools.translate import trans_load_data


class TestMailActivitiesCommon(TestTrialCommon):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.controller = main.Controller()
        with mute_logger('odoo.addons.base.models.ir_module', 'odoo.addons.base.models.ir_translation', 'odoo.tools.translate'):
            load_language(cls.env.cr, 'fr_BE')

    def create_mail_activity(self, record, activity_type_id, user_id):
        return self.env['mail.activity'].create({
            'res_id': record.id,
            'res_model_id': self.env['ir.model']._get_id(record._name),
            'activity_type_id': activity_type_id.id,
            'user_id': user_id.id,
        })


@tagged('post_install', '-at_install', 'post_saas_trial_creation')
class TestMailActivities(TestMailActivitiesCommon):
    """
    Unit tests for the retrieval of mail activities on the databases dashboard
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.activity_type_bill = cls.env['mail.activity.type'].create({'name': 'Bill clients'})
        cls.activity_type_pay = cls.env['mail.activity.type'].create({'name': 'Pay employees'})

        mat_bill_xmlid = cls.activity_type_bill.export_data(['id']).get('datas')[0][0]
        mat_pay_xmlid = cls.activity_type_pay.export_data(['id']).get('datas')[0][0]

        po_string = f'''
        #. module: __export__
        #: model:mail.activity.type,name:{mat_bill_xmlid}
        msgid "Bill clients"
        msgstr "Facturer les clients"

        #. module: __export__
        #: model:mail.activity.type,name:{mat_pay_xmlid}
        msgid "Pay employees"
        msgstr "Payer les employés"
        '''
        with io.BytesIO(bytes(po_string, encoding='utf-8')) as f:
            f.name = 'dummy'
            if version_gte("16.0"):
                translation_importer = TranslationImporter(cls.env.cr, verbose=True)
                translation_importer.load(f, 'po', 'fr_BE')
                translation_importer.save(overwrite=True)
            else:
                trans_load_data(cls.env.cr, f, 'po', 'fr_BE', verbose=True, overwrite=True)

    def setUp(self):
        super().setUp()
        self.user_employee.write({'oauth_uid': '-12345'})

        partner1 = self.env.ref('base.public_partner')
        self.mail_activity_todo1 = self.create_mail_activity(partner1, self.activity_type_bill, self.user_employee)
        self.mail_activity_todo2 = self.create_mail_activity(partner1, self.activity_type_bill, self.user_admin)
        self.mail_activity_todo3 = self.create_mail_activity(partner1, self.activity_type_pay, self.user_employee)
        self.mail_activity_todo4 = self.create_mail_activity(partner1, self.activity_type_pay, self.user_admin)

    def get_mail_activities(self):
        # as it is a SQL query, we need to flush the ORM cache to the transaction first
        if version_gte("saas~14.5"):
            self.env.cr.flush()
        elif version_gte("14.0"):
            self.env["base"].flush()

        return self.controller._get_mail_activities(self.env.cr, '-12345')

    def test_get_mail_activities(self):
        # Show mail activities associated to this employee
        self.assertEqual(self.get_mail_activities(), {'Bill clients': 1, 'Pay employees': 1})

        # Some activity types can be seen by all employees, whatever the associated user
        self.activity_type_bill.dashboard_visibility = 'all'
        self.assertEqual(self.get_mail_activities(), {'Bill clients': 2, 'Pay employees': 1})

        # Make sure that we use the employee's language even for all selected activities
        self.user_employee.lang = 'fr_BE'
        self.assertEqual(self.get_mail_activities(), {'Facturer les clients': 2, 'Payer les employés': 1})


@tagged('post_install', '-at_install', 'post_accounting_saas_trial_creation')
class TestAvailableVatReturns(TestMailActivitiesCommon):
    def setUp(self):
        super().setUp()

        mat_reminder_xmlid = 'account_reports_tax_reminder.mail_activity_type_tax_report_to_be_sent'
        self.activity_type_reminder = self.env.ref(mat_reminder_xmlid, False)
        if not self.activity_type_reminder:
            self.skipTest("'account_reports_tax_reminder' is not installed")

        self.user_employee.write({
            'oauth_uid': '-12345',
            'groups_id': [(4, self.ref('account.group_account_user'), 0)],
        })

        self.company1 = self.env.ref('base.main_company')
        self.company1.country_id = self.env.ref('base.be')
        if hasattr(self.company1, '_get_and_update_tax_closing_moves'):
            # >= saas-14.3
            self.tax_closing_move1 = self.company1._get_and_update_tax_closing_moves(fields.Date.to_date('2023-03-15'),
                                                                                     include_domestic=True)
        elif hasattr(self.company1, '_create_edit_tax_reminder'):
            # >= saas-13.5
            self.tax_closing_move1 = self.company1._create_edit_tax_reminder(fields.Date.to_date('2023-03-15'))

        self.mail_activity_reminder1 = self.create_mail_activity(self.tax_closing_move1,
                                                                 self.activity_type_reminder,
                                                                 self.user_employee)

    def get_available_vat_returns(self):
        # as it is a SQL query, we need to flush the ORM cache to the transaction first
        if version_gte("saas~14.5"):
            self.env.cr.flush()
        elif version_gte("14.0"):
            self.env["base"].flush()

        return self.controller._list_available_vat_returns(self.env.cr, '-12345')

    def test_list_available_vat_returns(self):
        mod = self.env['ir.module.module'].search([('name', '=', 'account_reports_tax_reminder')])
        if not mod:
            self.skipTest("module 'account_reports_tax_reminder' is not available in this version")
        self.assertEqual(mod.state, 'installed', "module 'account_reports_tax_reminder' should be installed")

        self.assertEqual(self.get_available_vat_returns(), {
            'companies': {
                self.company1.id: {'name': self.company1.name, 'country_code': 'BE'},
            },
            'country_names': {
                'BE': 'Belgium',
            },
            'reports': [
                {'company_id': self.company1.id, 'id': self.tax_closing_move1.id, 'period': '2023-03-31'},
            ],
            'result': True,
        })
